unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Spin, Menus, ShellAPI;

const
  MY_MESSAGE = WM_USER + 98345;

type
  //      
  TNotifyIconData = record
    cbSize: dword;               //  
    wnd: HWND;                   //  
    uID: uint;                   //  
    uFlags: uint;                //    
    uCallbackMsg: uint;          //  
    hIcon: HICON;                //  
    szTip: array[0..63] of char; // 
  end;

type
  TMainForm = class(TForm)
    ExitBtn: TButton;
    HideBtn: TButton;
    Timer: TTimer;
    MinSpinEdit: TSpinEdit;
    HourSpinEdit: TSpinEdit;
    ResetBtn: TButton;
    Label1: TLabel;
    Shape1: TShape;
    PopupMenu: TPopupMenu;
    mReset: TMenuItem;
    mShow: TMenuItem;
    N1: TMenuItem;
    StopBtn: TButton;
    N2: TMenuItem;
    mAbout: TMenuItem;
    MenuCloseTimer: TTimer;
    procedure ExitBtnClick(Sender: TObject);
    procedure HideBtnClick(Sender: TObject);
    procedure mResetClick(Sender: TObject);
    procedure mShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ResetBtnClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure HourSpinEditChange(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure mAboutClick(Sender: TObject);
    procedure MenuCloseTimerTimer(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
  private
    NID: TNotifyIconData;
    IconPresent: boolean;
    StartTime: TDateTime;
    BellTime: array[1..6] of TDateTime;
    Bells: array[1..6] of word;
    EndTime: TDateTime;
    Belled: boolean;
    procedure DoBell;
    procedure Delay(msec: word);
    procedure WndProc(var Msg: TMessage); override;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  RegEx, About;


// 1   ,  0
function IsLeapYear(Year: word): word;
begin
  if (Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0)) then Result := 1
  else Result := 0;
end;

function DaysPerMonth(Year,Month: word): word;
const
  DaysInMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
  Result := DaysInMonth[Month];
  if (Month = 2) then Result := Result + IsLeapYear(Year);
end;

function AddTime(Time: TDateTime; Hour,Min,Sec,mSec: word): TDateTime;
var
  Year,Month,Day,iHour,iMin,iSec,imSec: word;
begin
  DecodeDate(Time,Year,Month,Day);
  DecodeTime(Time,iHour,iMin,iSec,imSec);
  imSec := imSec + mSec;
  if imSec > 999 then begin
    imSec := imSec - 1000;
    Inc(iSec);
  end;
  iSec := iSec + Sec;
  if iSec > 59 then begin
    iSec := iSec - 60;
    Inc(iMin);
  end;
  iMin := iMin + Min;
  if iMin > 59 then begin
    iMin := iMin - 60;
    Inc(iHour);
  end;
  iHour := iHour + Hour;
  if iHour > 23 then begin
    iHour := iHour - 24;
    Inc(Day);
  end;
  if Day > DaysPerMonth(Year,Month) then begin
    Day := Day - DaysPerMonth(Year,Month);
    Inc(Month);
  end;
  if Month > 12 then begin
    Month := Month - 12;
    Inc(Year);
  end;
  Result := EncodeDate(Year,Month,Day) + EncodeTime(iHour,iMin,iSec,imSec);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Belled := FALSE;
  Timer.Enabled := TRUE;
  IconPresent := FALSE;
  with NID do begin
    cbSize := SizeOf(NID);
    uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
    uID := 14628;
    wnd := Handle;
    hIcon := Application.Icon.Handle;
    szTip := 'Timer';
    uCallbackMsg := MY_MESSAGE;
  end;
  RegEx.regSetAuthorName('Alexey Denisov');
  HourSpinEdit.Value := regReadInteger('Hours',0);
  MinSpinEdit.Value := regReadInteger('Minutes',25);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Shell_NotifyIcon(NIM_DELETE,@NID);
  regWriteInteger('Hours',HourSpinEdit.Value);
  regWriteInteger('Minutes',MinSpinEdit.Value);
end;

procedure TMainForm.WndProc;
var
  CursorPos: TPoint;
begin
  GetCursorPos(CursorPos);
  if (Msg.Msg = MY_MESSAGE) and (Msg.LParam = WM_RBUTTONUP) then begin
    PopupMenu.Popup(CursorPos.X + 32,CursorPos.Y + 32);
  end;
  inherited WndProc(Msg);
end;

procedure TMainForm.ExitBtnClick(Sender: TObject);
begin
  AboutForm.Close;
  MainForm.Close;
end;

procedure TMainForm.HideBtnClick(Sender: TObject);
begin
  Visible := FALSE;
end;

procedure TMainForm.mResetClick(Sender: TObject);
begin
  ResetBtn.Click;
end;

procedure TMainForm.mShowClick(Sender: TObject);
begin
  Visible := TRUE;
end;

procedure TMainForm.ResetBtnClick(Sender: TObject);
var
  Hour,Min,X: word;
  i: word;
begin
  StartTime := Now;
  DecodeTime(StartTime,Hour,Min,X,X);
  X := 1 + MinSpinEdit.Value + HourSpinEdit.Value * 60;
  for i := 1 to 6 do begin
    Bells[i] := i;
    BellTime[7-i] := AddTime(StartTime,0,X-i,0,0);
  end;
  EndTime := AddTime(BellTime[6],0,3,0,0);
  Timer.Enabled := TRUE;
end;

procedure TMainForm.Delay;
var
  EndOfDelay: TDateTime;
begin
  EndOfDelay := AddTime(Now,0,0,0,mSec);
  repeat until Now > EndOfDelay;
end;

procedure TMainForm.DoBell;
begin
  Belled := TRUE;
  MessageBeep(0);
  Delay(500);
  Belled := FALSE;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
var
  i: word;
  n: TDateTime;
begin
  //    - 
  if Belled then Exit;
  //    - ,   
  if not IconPresent then begin
    Shell_NotifyIcon(NIM_ADD,@NID);
    IconPresent := TRUE;
    HideBtn.Click;
    ResetBtn.Click;
    Exit;
  end;
  n := Now;
  //  3   - 
  //  (       )
  if n > EndTime then begin
    ResetBtn.Click;
    Exit;
  end;
  for i := 5 downto 1 do begin
    if (Bells[i] > 0) and (n > BellTime[i]) then begin
      DoBell;
      Dec(Bells[i]);
    end;
  end;
  if (n > BellTime[6]) then DoBell;
end;

procedure TMainForm.HourSpinEditChange(Sender: TObject);
begin
  if HourSpinEdit.Value > 0 then MinSpinEdit.MinValue := 0
  else with MinSpinEdit do begin
    MinValue := 5;
    if MinValue > Value then Value := MinValue;
  end;
end;

procedure TMainForm.StopBtnClick(Sender: TObject);
begin
  Timer.Enabled := FALSE;
end;

procedure TMainForm.mAboutClick(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TMainForm.MenuCloseTimerTimer(Sender: TObject);
var
  Wnd: HWND;
begin
  Wnd := GetCapture;
  if Wnd <> 0 then begin
    SendMessage(Wnd,WM_CANCELMODE,0,0);
  end;
end;

procedure TMainForm.PopupMenuPopup(Sender: TObject);
begin
  MenuCloseTimer.Enabled := FALSE;
  MenuCloseTimer.Enabled := TRUE;
end;

end.
